// Support for blog rooms
//
// Copyright (c) 1999-2024 by the citadel.org team
//
// This program is open source software.  Use, duplication, or disclosure
// is subject to the terms of the GNU General Public License, version 3.

#include "../../sysdep.h"
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <pwd.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <time.h>
#include <sys/wait.h>
#include <string.h>
#include <limits.h>
#include <libcitadel.h>
#include "../../citadel_defs.h"
#include "../../server.h"
#include "../../citserver.h"
#include "../../support.h"
#include "../../config.h"
#include "../../control.h"
#include "../../user_ops.h"
#include "../../database.h"
#include "../../msgbase.h"
#include "../../internet_addressing.h"
#include "../../serv_vcard.h"
#include "../../citadel_ldap.h"
#include "../../ctdl_module.h"

// Pre-save hook for saving a message in a blog room.
// (Do we want to only do this for top-level messages?)
int blog_upload_beforesave(struct CtdlMessage *msg, struct recptypes *recp) {

	// Only run this hook for blog rooms
	if (CC->room.QRdefaultview != VIEW_BLOG) {
		return(0);
	}

	// If the message doesn't have an EUID, give it one.
	if (CM_IsEmpty(msg, eExclusiveID)) {
		char uuid[SIZ];
		generate_uuid(uuid);
		CM_SetField(msg, eExclusiveID, uuid);
	}

	// We also want to define a maximum length, whether we generated it or not.
	CM_CutFieldAt(msg, eExclusiveID, BLOG_EUIDBUF_SIZE - 1);
	
	// Now allow the save to complete.
	return(0);
}


// Initialization function, called from modules_init.c
char *ctdl_module_init_blog(void) {
	if (!threading) {
		CtdlRegisterMessageHook(blog_upload_beforesave, EVT_BEFORESAVE);
	}
	
	// return our module id for the log
	return "blog";
}