// use table handler
//
// Copyright (c) 1987-2023 by the citadel.org team
//
// This program is open source software.  Use, duplication, or disclosure
// is subject to the terms of the GNU General Public License, version 3.


#include <stdio.h>
#include <libcitadel.h>

#include "citserver.h"
#include "ctdl_module.h"
#include "config.h"
#include "control.h"


// Has an item already been seen (is it in the CDB_USETABLE) ?
// Returns 0 if it hasn't, 1 if it has
// In either case, writes the item to the database for next time.
int CheckIfAlreadySeen(StrBuf *guid) {
	int found = 0;
	struct UseTable ut;
	struct cdbdata cdbut;
	int hash = HashLittle(ChrPtr(guid), StrLength(guid));

	syslog(LOG_DEBUG, "usetable: CheckIfAlreadySeen(0x%8x)", hash);
	cdbut = cdb_fetch(CDB_USETABLE, &hash, sizeof(hash));
	if (cdbut.ptr != NULL) {
		found = 1;
	}

	// (Re)write the record, to update the timestamp.
	ut.hash = hash;
	ut.timestamp = time(NULL);
	cdb_store(CDB_USETABLE, &hash, sizeof(hash), &ut, sizeof(struct UseTable));
	return(found);
}
