// This is a test utility for the backend API.  It simply opens and closes the databases.
// There is no need for end users to ever run this program.  So don't.  And don't ask.
//
// Copyright (c) 2023-2024 by Art Cancro citadel.org
//
// This program is open source software.  Use, duplication, or disclosure
// is subject to the terms of the GNU General Public License, version 3.

#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netdb.h>
#include <string.h>
#include <pwd.h>
#include <errno.h>
#include <stdarg.h>
#include <limits.h>
#include <syslog.h>
#include <libcitadel.h>
#include <zlib.h>
#include "../server/sysdep.h"
#include "../server/citadel_defs.h"
#include "../server/server.h"
#include "../server/citadel_dirs.h"
#include "../server/database.h"

uid_t ctdluid = 0;


int main(int argc, char **argv) {
	int i = 0;
	int confirmed = 0;
	char *ctdldir = CTDLDIR;

	// display the greeting
	fprintf(stderr,	"\033[44m\033[1m"
		"╔════════════════════════════════════════════════════════════════════════╗\n"
		"║ dbnothing utility for Citadel                                          ║\n"
		"║ Copyright (c) 2023-2024 by citadel.org et al.                          ║\n"
		"║ This program is open source software.  Use, duplication, or disclosure ║\n"
		"║ is subject to the terms of the GNU General Public license v3.          ║\n"
		"╚════════════════════════════════════════════════════════════════════════╝\033[0m\n"
	);

	// Parse command line
	int a;
	while ((a = getopt(argc, argv, "h:y")) != EOF) {
		switch (a) {
		case 'h':
			ctdldir = optarg;
			break;
		default:
			fprintf(stderr, "%s: usage: %s -s citadel_dir [>dumpfile]\n", argv[0], argv[0]);
			exit(2);
		}
	}

	if (chdir(ctdldir) != 0) {
		fprintf(stderr, "ctdlload: unable to change directory to [%s]: %m", ctdldir);
		exit(2);
	}

	// backend modules use syslog -- redirect to stderr
	openlog("dbnothing", LOG_PERROR , LOG_DAEMON);

	// initialize the database backend
	cdb_init_backends();
	cdb_open_databases();

	// do nothing
	fprintf(stderr, "\ndbnothing: doing nothing\n\n");

	// close databases
	cdb_close_databases();

	fprintf(stderr, "dbnothing: \033[32m\033[1mfinished\033[0m\n");
	exit(0);
}
